<?php

function bot($method, $datas = [])
{
    $url = "https://api.telegram.org/bot" . API_KEY . "/" . $method;

    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, $url);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    curl_setopt($ch, CURLOPT_POST, count($datas));
    curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($datas));

    $result = curl_exec($ch);

    if (curl_errno($ch)) {
        echo 'Curl error: ' . curl_error($ch);
    }

    curl_close($ch);

    return json_decode($result);
}

function sendMessage($datas = [])
{
    $result = bot("sendMessage", $datas);

    if (isset($result->ok) && $result->ok === true) {
        return $result->result->message_id;
    } else {
        error_log("Failed to send message: " . json_encode($result));
        return false;
    }
}

function editMessageText($datas = [])
{
    bot("editMessageText", $datas);
}

function answerCallbackQuery($datas = [])
{
    bot("answerCallbackQuery", $datas);
}

function deleteMessage($chat_id, $message_id)
{
    bot("deleteMessage", ['chat_id' => $chat_id, 'message_id' => $message_id]);
}

function cleanInput($data)
{
    return htmlspecialchars(strip_tags(trim($data)));
}

function saveBotMessage($chat_id, $message_id)
{
    global $conn;
    $sql = "INSERT INTO auto_delete_messages (chat_id, message_id, delete_time) VALUES (?, ?, ?)";
    $stmt = $conn->prepare($sql);

    if (!$stmt) {
        error_log("SQL Error: " . $conn->error);
        return false;
    }

    $delete_time = date("Y-m-d H:i:s", time() + 300);
    $stmt->bind_param("iis", $chat_id, $message_id, $delete_time);

    if (!$stmt->execute()) {
        error_log("Execution Error: " . $stmt->error);
        return false;
    }

    return true;
}

function cleanText($text)
{
    $text = preg_replace('/[^\p{L}\p{N}]/u', ' ', $text);
    $text = preg_replace('/\s+/', ' ', $text);
    return trim($text);
}

$persian_to_english_digits = array(
    '۰' => '0',
    '۱' => '1',
    '۲' => '2',
    '۳' => '3',
    '۴' => '4',
    '۵' => '5',
    '۶' => '6',
    '۷' => '7',
    '۸' => '8',
    '۹' => '9'
);

function replace_persian_digits($text)
{
    global $persian_to_english_digits;
    return str_replace(array_keys($persian_to_english_digits), array_values($persian_to_english_digits), $text);
}

function getNonJoinedChannels($user_id)
{
    $channelsToCheck = ["@qpcod", "@qpBots"];
    $notJoinedChannels = [];

    foreach ($channelsToCheck as $channel) {
        $response = bot("getChatMember", [
            'chat_id' => $channel,
            'user_id' => $user_id
        ]);

        if ($response->ok && $response->result->status == "left") {
            $notJoinedChannels[] = $channel;
        }
    }

    return $notJoinedChannels;
}

function beautifyPrice($price)
{
    // اطمینان از تبدیل مقدار ورودی به float
    $price = floatval($price);

    if ($price == 0) {
        return "0";
    } elseif ($price < 0.001) {
        // قیمت‌های خیلی کوچک: نمایش ۸ رقم اعشار
        return number_format($price, 8, '.', ',');
    } elseif ($price < 1) {
        // قیمت‌های کوچک: نمایش ۶ رقم اعشار
        return number_format($price, 6, '.', ',');
    } elseif ($price < 100) {
        // قیمت‌های متوسط: نمایش 4 رقم اعشار
        return number_format($price, 4, '.', ',');
    } elseif ($price < 1000) {
        // قیمت‌های بالا: نمایش 2 رقم اعشار
        return number_format($price, 2, '.', ',');
    } else {
        // قیمت‌های بسیار بالا: نمایش بدون رقم اعشار (یا می‌توانید 2 رقم هم انتخاب کنید)
        return number_format($price, 0, '.', ',');
    }
}

function getPriceFromBitpin($symbol, $count)
{

    $bitpin = json_decode(file_get_contents('bitpin/data.json'), true);
    $bitpinTokens = $bitpin['data'];

    foreach ($bitpinTokens as $bitpinToken) {

        if ($bitpinToken['symbol'] === $symbol) {

            $bitpinPrice = $bitpinToken['price'];

            $lastPrice = $bitpinPrice;
            $totalPrice = $bitpinPrice * $count;

            $formattedLastPrice = beautifyPrice($lastPrice);
            $formattedTotalPrice = beautifyPrice($totalPrice);

            return [
                'last_price' => $formattedLastPrice,
                'total_price' => $formattedTotalPrice,
                'currency' => "تومان",
                'urls' =>
                [
                    "ورود به صرافی بیت پین 💰" => "https://bitpin.ir/signup/?ref=3XmFr0nB",
                ]
            ];
        }
    }

    return false;
}

function getPriceFromToobit($symbol, $count)
{

    $toobit = json_decode(file_get_contents('toobit/data.json'), true);
    $toobitTokens = $toobit['data'];

    foreach ($toobitTokens as $toobitToken) {

        if ($toobitToken['s'] === $symbol) {

            $toobitPrice = $toobitToken['p'];

            $lastPrice = $toobitPrice;
            $totalPrice = $toobitPrice * $count;

            $formattedLastPrice = beautifyPrice($lastPrice);
            $formattedTotalPrice = beautifyPrice($totalPrice);

            return [
                'last_price' => $formattedLastPrice,
                'total_price' => $formattedTotalPrice,
                'currency' => "تتر",
                'urls' =>
                [
                    "آموزش توبیت 📝" => "https://youtu.be/iruqL5L6vUI",
                    "ورود به توبیت 💫" => "https://www.toobit.com/t/qpcode",
                ]
            ];
        }
    }

    return false;
}

function getPriceFromNobitex($symbol, $count, $rls = false)
{

    $tickers = json_decode(file_get_contents('nobitex/data.json'), true);

    $Tokens = $tickers['data']['stats'];

    if (isset($Tokens[$symbol])) {
        $lastPrice = $Tokens[$symbol]['latest'] / 10;

        if ($rls) {
            $lastPrice = $Tokens[$symbol]['latest'];
        }
        $totalPrice = $lastPrice * $count;

        $formattedLastPrice = beautifyPrice($lastPrice);
        $formattedTotalPrice = beautifyPrice($totalPrice);

        return [
            'last_price' => $formattedLastPrice,
            'total_price' => $formattedTotalPrice,
            'currency' => "تومان",
            'urls' =>
            [
                "آموزش نوبیتکس 📝" => "https://www.youtube.com/watch?v=ro-WsxeesG8",
                "ورود به نوبیتکس 💫" => "https://nobitex.ir/signup/?refcode=9270142",
            ]
        ];
    }

    return false;
}

function getPriceFromBitget($symbol, $count)
{

    $bitget = json_decode(file_get_contents('bitget/data.json'), true);
    $bitgetTokens = $bitget['data']['data'];

    foreach ($bitgetTokens as $bitgetToken) {

        if ($bitgetToken['symbol'] === $symbol) {

            $bitgetPrice = $bitgetToken['lastPr'];
            $change24h = $bitgetToken['change24h'];

            $lastPrice = $bitgetPrice;
            $totalPrice = $bitgetPrice * $count;

            $formattedLastPrice = beautifyPrice($lastPrice);
            $formattedTotalPrice = beautifyPrice($totalPrice);

            return [
                'last_price' => $formattedLastPrice,
                'total_price' => $formattedTotalPrice,
                'change_24' => $change24h,
                'currency' => "تتر",
                ];
        }
    }

    return false;
}

function addUserRequest($user_id)
{
    global $conn;
    // 1. بررسی وجود کاربر
    $sql    = "SELECT user_id FROM users WHERE user_id = ?";
    $stmt   = $conn->prepare($sql);
    $stmt->bind_param("i", $user_id);
    $stmt->execute();
    $result = $stmt->get_result();

    // 2. اگر کاربر در دیتابیس نبود، ثبت کنیم
    if ($result->num_rows === 0) {
        $insertSql = "INSERT INTO users (user_id, requests_count) VALUES (?, 0)";
        $insertStmt = $conn->prepare($insertSql);
        $insertStmt->bind_param("i", $user_id);
        $insertStmt->execute();
        $insertStmt->close();
    }

    // 3. افزایش تعداد درخواست‌ها
    $updateSql = "UPDATE users SET requests_count = requests_count + 1 WHERE user_id = ?";
    $updateStmt = $conn->prepare($updateSql);
    $updateStmt->bind_param("i", $user_id);
    $updateStmt->execute();
    $updateStmt->close();
}

function addGroupRequest($chat_id)
{
    global $conn;
    // 1. بررسی وجود گروه
    $sql    = "SELECT chat_id FROM groups WHERE chat_id = ?";
    $stmt   = $conn->prepare($sql);
    $stmt->bind_param("i", $chat_id);
    $stmt->execute();
    $result = $stmt->get_result();

    // 2. اگر کاربر در دیتابیس نبود، ثبت کنیم
    if ($result->num_rows === 0) {
        $join_date = date("Y-m-d H:i:s");
        $insertSql = "INSERT INTO groups (chat_id, 	join_date) VALUES (?, ?)";
        $insertStmt = $conn->prepare($insertSql);
        $insertStmt->bind_param("is", $chat_id, $join_date);
        $insertStmt->execute();
        $insertStmt->close();
    }

    // 3. افزایش تعداد درخواست‌ها
    $updateSql = "UPDATE groups SET requests_count = requests_count + 1 WHERE chat_id = ?";
    $updateStmt = $conn->prepare($updateSql);
    $updateStmt->bind_param("i", $chat_id);
    $updateStmt->execute();
    $updateStmt->close();
}

function searchWithSuggestions($query, $count)
{

    $bitget = json_decode(file_get_contents('bitget/data.json'), true);
    $bitgetTokens = $bitget['data']['data'];
    $symbols = [];

    foreach ($bitgetTokens as $bitgetToken) {
        $symbol =  $bitgetToken['symbol'];
        // چک می‌کنیم که 4 کاراکتر آخر نماد برابر با "USDT" باشد
        if (substr($symbol, -4) === 'USDT') {
            // "USDT" را حذف کرده و به آرایه اضافه می‌کنیم
            $symbols[] = substr($symbol, 0, -4);
        }
    }

    $result =  [
        "text" => "هیچ نتیجه‌ای برای نماد '$query' یافت نشد.",
        "ok" => false,
        "exchanges" => [
            "بیت گت" => false,
            "بیت پین" => false,
            "نوبیتکس" => false,
            "توبیت" => false,
        ],
        "suggestions" => []
    ];

    // تعریف آرایه نگاشت نام‌های مستعار به نام اصلی ارز
    $aliasMap = [
        'notcoin'  => 'NOT',
        'نات'      => 'NOT',
        'ناتکوین' => 'NOT',
        'نات کوین' => 'NOT',
        'داگز' => 'DOGS',
        'تون' => 'TON',
        'تان' => 'TON',
        'تون کوین' => 'TON',
        'تان کوین' => 'TON',
        'تن' => 'TON',
        'تن کوین' => 'TON',
        'بیتکوین' => 'BTC',
        'بیت کوین' => 'BTC',
        'bitcoin' => 'BTC',
        'solana' => 'SOL',
        'سولانا' => 'SOL',
        'ریپل' => 'XRP',
        'اتریوم' => 'ETH',
        'زوو' => 'ZOO',
        'زو' => 'ZOO',
        'میمهش' => 'MEMHASH',
        'میم هش' => 'MEMHASH',
        'تپس' => "TAPS",
        'تپ سواپ' => "TAPS",
        'تپسواپ' => "TAPS",
        'همستر' => "HMSTR",
        'hamster' => "HMSTR",
        'hamester' => "HMSTR",
        'همستر کامبت' => "HMSTR",
        'همستر کمبت' => "HMSTR",
        'پیکسل ورس' => "PIXFI",
        'میجر' => "MAJOR",
        'ماجور' => "MAJOR",
        'ماژور' => "MAJOR",
        'راکی' => "RBTC",
        'راکی رابیت' => "RBTC",
        'راکی ربیت' => 'RBTC',
        'سون' => 'SOON',
        'ایکس' => 'X',
        'ایکس ایمپایر' => 'X',
        'ترامپ' => 'TRUMP',
        'ترون' => 'TRX',
        'شیبا' => 'SHIB',
        'کتیزن' => 'CATI',
        'کتز' => 'CATS',
        'کتس' => 'CATS',
        'کت' => 'CATS',
        'گربه' => 'CATS',
        'وات' => 'WAT',
        'میمفای' => 'MEMEFI',
        'ممفای' => 'MEMEFI',
        'بایننس کوین' => 'BNB',
        'سونیک' => 'S',
        'هایپرلیوکویید' => 'HYPE',
        'هایپر لیوکویید' => 'HYPE',
        'هایپر لیوکوید' => 'HYPE',
        'پالیگان' => 'POL',
        'پولیگان' => 'POL',
        'آربیتروم' => 'ARB',
        'گرس' => 'GRASS',
        'رندر' => 'RENDER',
        'پپه' => 'PEPE',
        'دوج' => 'DOGE',
        'دوج کوین' => 'DOGE',
        'بیبی دوج' => 'BABYDOGE',
        'کاردانو' => 'ADA',
        'آدا' => 'ADA',
        'ادا' => 'ADA',
        'توما' => 'TOMA',
        'تومارکت' => 'TOMA',
        'tomarket' => 'TOMA',
        'بونک' => 'BONK',
        // --- ارزهای معروف ---
        'لایت کوین'      => 'LTC',
        'لایتکوین'       => 'LTC',
        'لایت'           => 'LTC',
        'پولکادات'       => 'DOT',
        'پولکا دات'      => 'DOT',
        'دات'            => 'DOT',
        'چین لینک'       => 'LINK',
        'لینک'           => 'LINK',
        'آوالانچ'        => 'AVAX',
        'اوالانچ'        => 'AVAX',
        'فانتوم'         => 'FTM',
        'فانتوم کوین'    => 'FTM',
        'تتا'            => 'THETA',
        'تزوس'           => 'XTZ',
        'هدرا'           => 'HBAR',
        'هدرا هشگرف'     => 'HBAR',
        'الگوراند'       => 'ALGO',
        'الگوراندو'      => 'ALGO', // در بعضی تلفظ‌ها
        'ویچین'          => 'VET',
        'وی چین'         => 'VET',
        'کازماس'         => 'ATOM',
        'کازمس'          => 'ATOM',
        'اتم'            => 'ATOM',
        'نیر'            => 'NEAR',
        'استلار'         => 'XLM',
        'استلار لومنز'   => 'XLM',
        'مونرو'          => 'XMR',
        'دش'             => 'DASH',
        'زد کش'          => 'ZEC',
        'بی تی تی'        => 'BTT',
        'بیت تورنت'      => 'BTT',
        'فایل کوین'      => 'FIL',
        'اپتیمیسم'       => 'OP',
        'اپتوس'          => 'APT',
        'اپتوس کوین'     => 'APT',
        'ایپ کوین'       => 'APE',
        'اپ کوین'        => 'APE',
        'سندباکس'        => 'SAND',
        'دیسنترالند'     => 'MANA',
        'انجین'          => 'ENJ',
        'انجین کوین'     => 'ENJ',
        'یونی سواپ'      => 'UNI',
        'پنکیک سواپ'     => 'CAKE',
        'سوشی سواپ'      => 'SUSHI',
        'کرو'            => 'CRV',
        'کوانت'          => 'QNT',
        'بیت کوین کش'    => 'BCH',
        'بیتکویین کش'    => 'BCH',
        'کروانوس'        => 'CRO',
        'کرونوس'         => 'CRO',
        'گالا'           => 'GALA',
        'سوی'            => 'SUI',
        'سویی'           => 'SUI',
        'میکر'           => 'MKR',
        'اوه'            => 'AAVE',
        'اور'            => 'AAVE',   // بعضاً اینطور تلفظ می‌شود
        'ای اجیکس'       => 'AGIX',   // تلفظ: "ای-جیکس" برای SingularityNET
        'اییجکس'         => 'AGIX',
        'سینگولاریتی نت' => 'AGIX',
        'فلوکی'          => 'FLOKI',
        'فلوکی اینو'     => 'FLOKI',
        'دی وای دی ایکس' => 'DYDX',
        'اینچکتیو'       => 'INJ',    // تلفظ نزدیک به "اینجکتیو"
        'کاوا'           => 'KAVA',
        'جی ام ایکس'     => 'GMX',
        'بون شیبا'       => 'BONE',
        'چیلیز'         => 'CHZ',
        'سلو'            => 'CELO',
        'پکس گلد'        => 'PAXG',
        'ایکس دی سی'     => 'XDC',
        'ایکس دی سی نتورک' => 'XDC',
        'x empier' => 'X',
        'پنگو' => 'PENGU',
        'پای' => 'PI',
        'پای نتورک' => 'PI',
    ];

    $userInput = $query;
    $userInputLower = mb_strtolower($userInput, 'UTF-8');

    // نگاشت به نام اصلی در صورت وجود
    if (isset($aliasMap[$userInputLower])) {
        $query = $aliasMap[$userInputLower];
    } else {
        $query = strtoupper($userInput);
    }

    // 1) اگر جستجوی دقیق وجود دارد
    if (in_array($query, $symbols)) {

        $result =  [
            "text" => "نماد '$query' پیدا شد!",
            "ok" => true,
            "symbol" => $query,
            "exchanges" => [
                "بیت گت" => getPriceFromBitget("$query" . "USDT", $count),
                "بیت پین" => getPriceFromBitpin($query . "_IRT", $count),
                "نوبیتکس" => getPriceFromNobitex(strtolower($query) . "-rls", $count),
                "توبیت" => getPriceFromToobit($query . "USDT", $count),
            ]
        ];

        return $result;
    }

    // 2) بررسی اینکه آیا عبارت وارد شده، ابتدای بعضی نمادها هست یا خیر
    $partialMatches = [];
    foreach ($symbols as $symbol) {
        if (strpos($symbol, $query) != false) {
            $partialMatches[] = $symbol;
        }
    }
    if (!empty($partialMatches)) {
        $partialMatches = array_slice($partialMatches, 0, 10);
        $result['text'] = "هیچ رمزارزی با نماد '$query' پیدا نشد، اما این موارد با ابتدای '$query' مطابقت دارند: ";
        $result['suggestions'] = $partialMatches;
        return $result;
    }

    // 2) بررسی اینکه آیا عبارت وارد شده، ابتدای بعضی نمادهای فارسی هست یا خیر
    $partialMatches = [];
    foreach (array_keys($aliasMap) as $symbol) {
        if (mb_strpos($symbol, $query, 0, 'UTF-8') !== false) {
            $partialMatches[] = $symbol;
        }
    }
    if (!empty($partialMatches)) {
        $partialMatches = array_slice($partialMatches, 0, 10);
        $result['text'] = "هیچ رمزارزی با نماد '$query' پیدا نشد، اما این موارد مطابقت دارند: ";
        $result['suggestions'] = $partialMatches;
        return $result;
    }

    // 3) اگر هنوز چیزی نیافتیم، یکی یکی حروف انتهایی را حذف می‌کنیم تا به نتیجه برسیم
    $temp = $query;
    $suggestions = [];
    while (strlen($temp) > 0) {
        $temp = substr($temp, 0, -1);  // حذف آخرین کاراکتر

        // اگر خالی شد، دیگر چیزی نداریم
        if ($temp === '') {
            break;
        }

        foreach ($symbols as $symbol) {
            if (strpos($symbol, $temp) != false || strtoupper($symbol) == $temp) {
                $suggestions[] = $symbol;
            }
        }

        foreach (array_keys($aliasMap) as $symbol) {
            if (mb_strpos($symbol, $temp, 0, 'UTF-8') !== false) {
                $suggestions[] = $symbol;
            }
        }

        // اگر با همین کوتاه کردن نتایجی به دست آمد، متوقف شو
        if (!empty($suggestions)) {
            break;
        }
    }

    if (!empty($suggestions)) {
        $suggestions = array_slice($suggestions, 0, 10);
        $result['text'] = "هیچ رمزارزی با نماد '$query' پیدا نشد. آیا منظور شما یکی از موارد زیر است؟ ";
        $result['suggestions'] = $suggestions;
        return $result;
    }

    // 3) اگر هنوز چیزی نیافتیم، یکی یکی حروف انتهایی را حذف می‌کنیم تا به نتیجه برسیم
    $temp = $query;
    $suggestions = [];

    while (strlen($temp) > 0) {
        $temp = substr($temp, 0, -1);  // حذف آخرین کاراکتر

        // اگر خالی شد، دیگر چیزی نداریم
        if ($temp === '') {
            break;
        }

        foreach (array_keys($aliasMap) as $symbol) {
            if (substr($symbol, 0, strlen($temp)) === $temp) {
                $suggestions[] = $symbol;
            }
        }

        // اگر با همین کوتاه کردن نتایجی به دست آمد، متوقف شو
        if (!empty($suggestions)) {
            break;
        }
    }

    if (!empty($suggestions)) {
        $suggestions = array_slice($suggestions, 0, 10);
        $result['text'] = "هیچ رمزارزی با نماد '$query' پیدا نشد. آیا منظور شما یکی از موارد زیر است؟ ";
        $result['suggestions'] = $suggestions;
        return $result;
    } else {
        $result['text'] = "نماد '$query' اصلا پیدا نشد!";
        $result['ok'] = null;
        return $result;
    }
}
